@@ -1,6 +1,6 @@
 package ExtUtils::Constant;
 use vars qw (@ISA $VERSION @EXPORT_OK %EXPORT_TAGS);
-$VERSION = '0.23_03';
+$VERSION = '0.23_04';
 $VERSION = eval $VERSION;
 
 =head1 NAME
@@ -456,6 +456,19 @@ as detailed in L<"C_constant">.
 =item PROXYSUBS
 
 If true, uses proxy subs. See L<ExtUtils::Constant::ProxySubs>.
+Options: autoload, push, croak_on_error or croak_on_read with the
+options being exclusive.
+
+Warning: Code generated via ProxySubs with any of the options push,
+croak_on_error or croak_on_read will only work for newer Perl versions
+and thus should not be used for CPAN modules.
+push can only be used with >= 5.10,
+croak_on_error >= 5.14,
+croak_on_read >= 5.24,
+ProxySubs or with autoload >= 5.8.
+
+ExtUtils::Constant::ProxySubs versions older than 0.23_04 even creates
+code usable only >=5.14.
 
 =item C_FH
 
